@@ -157,9 +157,8 @@ class TestSubscriptionWorker:
     @pytest.fixture(scope="function")
     def worker(self, settings, nats_conn):
         writer = AsyncMock()
-        reader = AsyncMock()
         with mock.patch("nucliadb_node.pull.get_storage"):
-            worker = Worker(writer, reader, "node")
+            worker = Worker(writer, "node")
             worker.store_seqid = Mock()
             yield worker
 
