@@ -18,6 +18,7 @@
 // along with this program. If not, see <http://www.gnu.org/licenses/>.
 //
 
+use std::collections::HashMap;
 use std::fmt::Debug;
 use std::fs;
 use std::path::Path;
@@ -27,6 +28,7 @@ use nucliadb_core::prelude::*;
 use nucliadb_core::protos::resource::ResourceStatus;
 use nucliadb_core::protos::{Resource, ResourceId};
 use nucliadb_core::tracing::{self, *};
+use nucliadb_core::IndexFiles;
 use nucliadb_procs::measure;
 use tantivy::collector::Count;
 use tantivy::query::AllQuery;
@@ -36,12 +38,12 @@ use tantivy::{doc, Index, IndexSettings, IndexSortByField, IndexWriter, Order};
 use super::schema::{timestamp_to_datetime_utc, TextSchema};
 
 const TANTIVY_INDEX_ARENA_MEMORY: usize = 6_000_000;
-const MERGE_THRESHOLD: usize = 5;
 
 pub struct TextWriterService {
     index: Index,
     pub schema: TextSchema,
     writer: IndexWriter,
+    config: TextConfig,
 }
 
 impl Debug for TextWriterService {
@@ -142,24 +144,48 @@ impl WriterChild for TextWriterService {
         Ok(())
     }
 
-    #[measure(actor = "texts", metric = "garbage_collection")]
-    #[tracing::instrument(skip_all)]
     fn garbage_collection(&mut self) -> NodeResult<()> {
-        self.writer.garbage_collect_files()?;
         Ok(())
     }
 
-    #[measure(actor = "texts", metric = "merge")]
-    #[tracing::instrument(skip_all)]
-    fn merge(&mut self) -> NodeResult<()> {
-        let mut ids = self.index.searchable_segment_ids()?;
-        if ids.len() <= MERGE_THRESHOLD {
-            Ok(())
-        } else {
-            ids.truncate(MERGE_THRESHOLD);
-            self.writer.merge(&ids)?;
-            Ok(())
+    fn get_segment_ids(&self) -> NodeResult<Vec<String>> {
+        Ok(self
+            .index
+            .searchable_segment_ids()?
+            .iter()
+            .map(|s| s.uuid_string())
+            .collect())
+    }
+
+    fn get_index_files(&self, ignored_segment_ids: &[String]) -> NodeResult<IndexFiles> {
+        // Should be called along with a lock at a higher level to be safe
+        let mut meta_files = HashMap::new();
+        let path = self.config.path.join("meta.json");
+        meta_files.insert("text/meta.json".to_string(), fs::read(path)?);
+
+        let mut files = Vec::new();
+
+        for segment_meta in self.index.searchable_segment_metas()? {
+            if ignored_segment_ids.contains(&segment_meta.id().uuid_string()) {
+                continue;
+            }
+            for seg_file in segment_meta.list_files() {
+                files.push(format!("text/{}", seg_file.to_string_lossy()));
+            }
         }
+
+        if files.is_empty() {
+            // exit with no changes
+            return Ok(IndexFiles {
+                metadata_files: HashMap::new(),
+                files,
+            });
+        }
+
+        Ok(IndexFiles {
+            metadata_files: meta_files,
+            files,
+        })
     }
 }
 
@@ -178,12 +204,15 @@ impl TextWriterService {
     pub fn open(config: &TextConfig) -> NodeResult<Self> {
         let field_schema = TextSchema::new();
         let index = Index::open_in_dir(&config.path)?;
-        let writer = index.writer_with_num_threads(1, TANTIVY_INDEX_ARENA_MEMORY)?;
+        let writer = index
+            .writer_with_num_threads(1, TANTIVY_INDEX_ARENA_MEMORY)
+            .unwrap();
 
         Ok(TextWriterService {
             index,
             writer,
             schema: field_schema,
+            config: config.clone(),
         })
     }
 
@@ -204,12 +233,15 @@ impl TextWriterService {
         let index = index_builder
             .create_in_dir(&config.path)
             .expect("Index directory should exist");
-        let writer = index.writer_with_num_threads(1, TANTIVY_INDEX_ARENA_MEMORY)?;
+        let writer = index
+            .writer_with_num_threads(1, TANTIVY_INDEX_ARENA_MEMORY)
+            .unwrap();
 
         Ok(TextWriterService {
             index,
             writer,
             schema: field_schema,
+            config: config.clone(),
         })
     }
 
